/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkImageLineActor.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkImageLineActor - actor that draws a line on an image
// .SECTION Description
// vtkImageLineActor is an actor that draws a line and two end points in
// an image. Normally the image is 2D, but the widget will work in 3D as
// well. To use the widget, specify the image that the widget should refer
// to, as well as the end points (as an image location in i-j-k pixel
// coordinates and r-s-t subpixel position). Optionally, you can also
// specify the style of the end points by using an instance of the 
// vtkGlyph2D class.
//
// .SECTION Caveats
// While this actor is similar to a vtkActor2D, the way it is positioned is
// quite different. vtkActor2D is positioned using instances of vtkCoordinate
// for its Position and Position2 instance variables. (These specify position
// in a variety of different coordinate systems). Conversely, the positioning
// of vtkImageLineActor is based on specifying a referring vtkImageData,
// and the pixel locations in the image.

// .SECTION See Also
// vtkProp

#ifndef __vtkImageLineActor_h
#define __vtkImageLineActor_h

#include "vtkProp.h"
#include "vtkCoordinate.h" // For vtkViewportCoordinateMacro

class vtkImageData;
class vtkProperty2D;
class vtkMapper2D;

class VTK_WIDGETS_EXPORT vtkImageLineActor : public vtkProp
{
public:
  vtkTypeRevisionMacro(vtkImageLineActor,vtkProp);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Instantiate this class with Point1 located at the origin and
  // Point2 at (1,1,1, 0.0,0.0,0.0).
  static vtkImageLineActor *New();

  // Description:
  // Set/Get the image data input for the image actor.  
  virtual void SetInput(vtkImageData *);
  vtkGetObjectMacro(Input,vtkImageData);

  // Descrition:
  // Set/Get the i-j-k coordinates and the sub-pixel r-s-t values for the
  // two end points of the actor.
  void SetPoint1(vtkIdType ijk[3], double rst[3]);
  void GetPoint1(vtkIdType ijk[3], double rst[3]);
  void SetPoint2(vtkIdType ijk[3], double rst[3]);
  void GetPoint2(vtkIdType ijk[3], double rst[3]);

  // Description:
  // Specify a property to modify the appearance of this actor. Regarding
  // the GetProperty() method: a property will be automatically created
  // if one is not previously set.
  virtual void SetProperty(vtkProperty2D*);
  vtkProperty2D* GetProperty();
  
  // Description:
  // Get the position of the end points using an instance of vtkCoordinate.
  // The coordinates can be obtained in any coordinate system supported
  // by vtkCoordinate.
  vtkCoordinate *GetPoint1Coordinate()
    {return this->Point1Coordinate;}
  vtkCoordinate *GetPoint2Coordinate()
    {return this->Point2Coordinate;}

  // Description:
  // Return this objects modified time.
  unsigned long GetMTime();

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  virtual void ReleaseGraphicsResources(vtkWindow *);

//BTX
  // Description:
  // Support the standard render methods.
  int RenderOverlay(vtkViewport *viewport);
  int RenderOpaqueGeometry(vtkViewport *viewport);
  int RenderTranslucentGeometry(vtkViewport *viewport);
//ETX

protected:
   vtkImageLineActor();
  ~vtkImageLineActor();

  vtkImageData  *Input;
  int           MarkerStyle;
  vtkMapper2D   *Mapper;
  vtkProperty2D *Property;
  
  // Positioning information
  vtkIdType     P1Location[3];
  vtkIdType     P2Location[3];
  double        P1Subpixel[3];
  double        P2Subpixel[3];
  vtkCoordinate *Point1Coordinate;
  vtkCoordinate *Point2Coordinate;

private:
  vtkImageLineActor(const vtkImageLineActor&);  // Not implemented.
  void operator=(const vtkImageLineActor&);  // Not implemented.
};


#endif

