/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkSliderRepresentation.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkSliderRepresentation - provide the representation for a vtkSliderWidget
// .SECTION Description
// This class is used to represent and render a vtkSliderWidget. To use this
// class, you must at a minimum specify the end points of the
// slider. Optional instance variable can be used to modify the appearance of
// the widget.
//
// This class may be subclassed so that alternative representations can
// be created.  The class defines an API, and a default implementation, that
// the vtkSliderWidget interacts with to render itself in the scene.

// .SECTION See Also
// vtkSliderWidget


#ifndef __vtkSliderRepresentation_h
#define __vtkSliderRepresentation_h

#include "vtkWidgetRepresentation.h"
#include "vtkCoordinate.h" // For vtkViewportCoordinateMacro

class vtkActor;
class vtkPolyDataMapper;
class vtkSphereSource;
class vtkCellPicker;
class vtkProperty;
class vtkCylinderSource;
class vtkVectorText;
class vtkAssembly;
class vtkTransform;
class vtkTransformPolyDataFilter;
class vtkMatrix4x4;


class VTK_WIDGETS_EXPORT vtkSliderRepresentation : public vtkWidgetRepresentation
{
public:
  // Description:
  // Instantiate the class.
  static vtkSliderRepresentation *New();

  // Description:
  // Standard methods for the class.
  vtkTypeRevisionMacro(vtkSliderRepresentation,vtkWidgetRepresentation);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Position the first end point of the slider. Note that this point is an
  // instance of vtkCoordinate, meaning that Point 1 can be specified in a
  // variety of coordinate systems, and can even be relative to another
  // point. To set the point, you'll want to get the Point1Coordinate and
  // then invoke the necessary methods to put it into the correct coordinate
  // system and set the correct initial value.
  vtkCoordinate *GetPoint1Coordinate();

  // Description:
  // Position the second end point of the slider. Note that this point is an
  // instance of vtkCoordinate, meaning that Point 1 can be specified in a
  // variety of coordinate systems, and can even be relative to another
  // point. To set the point, you'll want to get the Point2Coordinate and
  // then invoke the necessary methods to put it into the correct coordinate
  // system and set the correct initial value.
  vtkCoordinate *GetPoint2Coordinate();

  // Description:
  // Specify whether to use a sphere or cylinder slider shape. By default, a
  // sphere shape is used.
  vtkSetClampMacro(SliderShape,int,SphereShape,CylinderShape);
  vtkGetMacro(SliderShape, int);
  void SetSliderShapeToSphere() { this->SetSliderShape(SphereShape); }
  void SetSliderShapeToCylinder() { this->SetSliderShape(CylinderShape); }
  
  // Description: 
  // Specify the length of the slider shape (in normalized coordinates
  // [0.01,0.5]). The slider length by default is 0.05.
  vtkSetClampMacro(SliderLength,double,0.01,0.5);
  vtkGetMacro(SliderLength,double);
  
  // Description:
  // Set the width of the slider in the directions orthogonal to the
  // slider axis. Using this it is possible to create ellipsoidal and hockey
  // puck sliders. By default the width is 0.05.
  vtkSetClampMacro(SliderWidth,double,0.0,1.0);
  vtkGetMacro(SliderWidth,double);
  
  // Description:
  // Specify the length of each end cap (in normalized coordinates
  // [0.0,0.25]). By default the length is 0.025. If the end cap length
  // is set to 0.0, then the end cap will not display at all.
  vtkSetClampMacro(EndCapLength,double,0.0,0.25);
  vtkGetMacro(EndCapLength,double);
  
  // Description:
  // Set the width of the tube (in normalized coordinates) in the directions
  // orthogonal to the tube axis. By default the relative scale is 0.025.
  vtkSetClampMacro(TubeWidth,double,0.01,0.25);
  vtkGetMacro(TubeWidth,double);
  
  // Description:
  // Set the rotation of the slider widget around the axis of the widget. This is
  // used to control which way the widget is initially oriented. (This is especially
  // important for the label and title.)
  vtkSetMacro(Rotation,double);
  vtkGetMacro(Rotation,double);

  // Description:
  // Get the slider properties. The properties of the slider when selected 
  // and unselected can be manipulated.
  vtkGetObjectMacro(SliderProperty,vtkProperty);
  
  // Description:
  // Get the properties for the tube and end caps. 
  vtkGetObjectMacro(TubeProperty,vtkProperty);
  vtkGetObjectMacro(CapProperty,vtkProperty);
  
  // Description:
  // Get the selection property. This property is used to modify the appearance of
  // selected objects (e.g., the slider).
  vtkGetObjectMacro(SelectedProperty,vtkProperty);
  
  // Description:
  // Specify the current value for the widget. The value should lie between
  // the minimum and maximum values.
  void SetValue(double value);
  vtkGetMacro(Value,double);
  
  // Description:
  // Set the current minimum value that the slider can take. Setting the
  // minimum value greater than the maximum value will cause the maximum
  // value to grow to (minimum value + 1).
  void SetMinimumValue(double value);
  vtkGetMacro(MinimumValue,double);
  
  // Description:
  // Set the current maximum value that the slider can take. Setting the
  // maximum value less than the minimum value will cause the minimum
  // value to change to (maximum value - 1).
  void SetMaximumValue(double value);
  vtkGetMacro(MaximumValue,double);
  
  // Description:
  // Specify the label text for this widget. If the value is not set, or set
  // to the empty string "", then the label text is not displayed.
  void SetTitleText(const char* label);
  const char* GetTitleText();

  // Description:
  // Specify the relative height of the label as compared to the length of the
  // slider. 
  vtkSetClampMacro(LabelHeight,double,0.0,2.0);
  vtkGetMacro(LabelHeight,double);

  // Description:
  // Specify the relative height of the title as compared to the length of the
  // slider. 
  vtkSetClampMacro(TitleHeight,double,0.0,2.0);
  vtkGetMacro(TitleHeight,double);

  // Description:
  // Indicate whether the slider text label should be displayed. This is
  // a number corresponding to the current Value of this widget.
  vtkSetMacro(ShowSliderLabel,int);
  vtkGetMacro(ShowSliderLabel,int);
  vtkBooleanMacro(ShowSliderLabel,int);

  // Description:
  // Methods to interface with the vtkSliderWidget.
  virtual void AddProps(vtkRenderer*);
  virtual void RemoveProps(vtkRenderer*);
  virtual void BuildRepresentation();
  virtual void StartWidgetInteraction(vtkRenderer*, double*);
  virtual void WidgetInteraction(vtkRenderer*, double*);
  virtual int ComputeInteractionState(vtkRenderer*, int, int);
  double GetPickedT();

//BTX
  // Enums define the state of the widget.
  enum _InteractionState
  {
    Outside=0,
    Tube,
    LeftCap,
    RightCap,
    Slider
  };
//ETX



protected:
  vtkSliderRepresentation();
  ~vtkSliderRepresentation();

  // Positioning the widget
  vtkCoordinate *Point1Coordinate;
  vtkCoordinate *Point2Coordinate;
  double        Length;

  // This is the slider end point coordinates in world coordinates
  double        P1[3];
  double        P2[3];

  // These are the slider end points taking into account the thickness
  // of the slider
  double        SP1[3];
  double        SP2[3];
  
  // Values
  double Value;  
  double MinimumValue;
  double MaximumValue;
  
  // More ivars controlling the appearance of the widget
  int    SliderShape;
  double SliderLength;
  double SliderWidth;
  double EndCapLength;
  double TubeWidth;
  double Rotation;

  // The current parametric coordinate
  double CurrentT;
  double PickedT;
  void EvaluateSliderPosition(double t, double position[3]);

  // Do the picking
  vtkCellPicker *Picker;
  void Highlight(int);

  // Determine the parameter t along the slider
  virtual double ComputePickPosition(vtkRenderer *ren, double eventPos[2]);

  // The widget consists of several actors, all grouped
  // together using an assembly. This makes it easier to
  // perform the final transformation into 
  vtkAssembly *WidgetAssembly;

  // Cylinder used by other objects
  vtkCylinderSource          *CylinderSource;
  vtkTransformPolyDataFilter *Cylinder;

  // The tube
  vtkPolyDataMapper *TubeMapper;
  vtkActor          *TubeActor;
  vtkProperty       *TubeProperty;
  
  // The slider
  vtkSphereSource   *SliderSource;
  vtkPolyDataMapper *SliderMapper;
  vtkActor          *SliderActor;
  vtkProperty       *SliderProperty;
  vtkProperty       *SelectedProperty;
  
  // The left cap
  vtkPolyDataMapper *LeftCapMapper;
  vtkActor          *LeftCapActor;
  vtkProperty       *CapProperty;

  // The right cap
  vtkPolyDataMapper *RightCapMapper;
  vtkActor          *RightCapActor;
  
  // The text. There is an extra transform used to rotate
  // both the title and label
  int               ShowSliderLabel;
  vtkVectorText     *LabelText;
  vtkPolyDataMapper *LabelMapper;
  vtkActor          *LabelActor;
  
  double            LabelHeight;
  double            TitleHeight;
  vtkVectorText     *TitleText;
  vtkPolyDataMapper *TitleMapper;
  vtkActor          *TitleActor;

  // Transform used during slider motion 
  vtkMatrix4x4 *Matrix;
  vtkTransform *Transform;

//BTX - manage the state of the widget
  enum _SliderShape {    
    SphereShape,
    CylinderShape
  };

//ETX


private:
  vtkSliderRepresentation(const vtkSliderRepresentation&);  //Not implemented
  void operator=(const vtkSliderRepresentation&);  //Not implemented
};

#endif
